--
-- MapBGASilo
-- This is the specialization for MapBGASilo
--
-- M@D Author  Headshot XXL
-- M@D date 18.07.2010
--
-- > Copyright (C) Headshot XXL - www.planet-ls.de < --


print("Map: BGASilo v1.0 loaded");


Event = {};

Event.silos = {};

function Event:loadMap(name)

	local dir = getUserProfileAppPath() .. "savegame"..gameMenuSystem.quickPlayMenu.selectedIndex;
	createFolder(dir);
    self.saveXMLFile = dir .. "/bgaSilo.xml";

	if g_missionLoaderDesc.stats.fuelUsage == 0 then
		os.remove(self.saveXMLFile);
		self.firstTimeRun = false;
	else
		--Event:loadBGAFromFile();
		self.firstTimeRun = true;
	end;
	

end;

function Event:deleteMap()
	
	Event.silos = {};

end;

function Event:delete()
end;

function Event:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Event:keyEvent(unicode, sym, modifier, isDown)
end;

function Event:update(dt)

	if self.firstTimeRun then
		Event:loadBGASiloFromFile();
		self.firstTimeRun = false;
	end;

	for i=1, table.getn(Event.silos) do
		Event.silos[i]:update(dt);
	end;

end;

function Event:draw()

	for i=1, table.getn(Event.silos) do
		Event.silos[i]:draw();
	end;
	
end;

local QuickPlayMenuSaveSelectedGame = QuickPlayMenu.saveSelectedGame;
QuickPlayMenu.saveSelectedGame = function(self)
	QuickPlayMenuSaveSelectedGame(self); 
	Event:saveBGASiloToXML();
	--print("bga saved")
end;

function Event:saveBGASiloToXML()	
	
	local existDir = io.open (self.saveXMLFile, "w");	
	if existDir == nil then
		createXMLFile("createsilo", self.saveXMLFile, "savesilo");
	end;

    -- save silo filllevel
    local xmlFile = io.open (self.saveXMLFile, "w");
    if xmlFile ~= nil then
        xmlFile:write('<?xml version="1.0" encoding="iso-8859-1" standalone="no" ?>\n<bgaSilos>\n');

		--xmlFile:write('    <silo fillLevel="'..self.fillLevel..'" siloMaxFillLevel="'..self.silo.maxFillLevel..'" state="'..self.siloState..'" planeTime="'..self.planeTime..'" />\n');

		for i=1, table.getn(Event.silos) do
			local silo = Event.silos[i];
			xmlFile:write('    <silo fillLevel="'..silo.fillLevel..'" siloMaxFillLevel="'..silo.silo.maxFillLevel..'" state="'..silo.siloState..'" />\n');
		end;		
		
        xmlFile:write("</bgaSilos>");

        xmlFile:close();
    end;	
	
end;

function Event:loadBGASiloFromFile()

	local existDir = io.open (self.saveXMLFile, "r");
	if existDir ~= nil then
		local xmlFile = loadXMLFile("TempConfig", self.saveXMLFile);
		
		local siloI = 0;
		while true do
			local key = string.format("bgaSilos.silo(%d)", siloI);
			local siloMaxFillLevel = getXMLFloat(xmlFile, key.."#siloMaxFillLevel");
			if siloMaxFillLevel ~= nil then			
				Event.silos[siloI+1].fillLevel = getXMLFloat(xmlFile, key.."#fillLevel");
				Event.silos[siloI+1].silo.maxFillLevel = siloMaxFillLevel;
				Event.silos[siloI+1].siloState = getXMLFloat(xmlFile, key.."#state");
				
				Event.silos[siloI+1]:updateState();
				
				Event.silos[siloI+1]:moveSilagePlane(Event.silos[siloI+1].fillLevel);
				Event.silos[siloI+1]:moveSiloPlane(Event.silos[siloI+1].fillLevel);
			else
				break;
			end;
			
			siloI = siloI +1;
		end;
		
		delete(xmlFile);
		print("BGASilo saves loaded");
	else
		print("Warning: ", self.saveXMLFile, " can not be found, BGASilo saves not loaded.")
	end;

end;

addModEventListener(Event);


MapBGASilo = {};

local MapBGASilo_mt = Class(MapBGASilo);

_G.BGASiloTipOnCreate = function(id)
	--print("created BGASilo trigger, id: ", id);
	table.insert(Event.silos, MapBGASilo:new(id));
end;

function MapBGASilo:new(id)

	local instance = {};
    setmetatable(instance, MapBGASilo_mt);

	addTrigger(id, "SiloTipCallback", instance);
	
	instance.silage = {};
	instance.silage.moveMaxY = getUserAttribute(id, "moveMaxY");
	instance.silage.moveMinY = getUserAttribute(id, "moveMinY");
	instance.silage.maxFillLevel = getUserAttribute(id, "maxFillLevel");
	instance.silage.plane = Utils.indexToObject(getParent(id), getUserAttribute(id, "silageIndex"));
	instance.silage.allowFillTypes = {};
	local types = Utils.splitString(" ", getUserAttribute(id, "fruitTypes"));
	for k,v in pairs(types) do
		if FruitUtil.fruitTypes[v] ~= nil then
			instance.silage.allowFillTypes[FruitUtil.fruitTypes[v].index] = true;
		end;
	end;
	
	
	instance.silo = {};
	instance.silo.plane = Utils.indexToObject(getParent(id), getUserAttribute(id, "siloIndex"));
	instance.silo.trigger = Utils.indexToObject(getParent(id), getUserAttribute(id, "siloTriggerIndex"));
	instance.silo.triggerNode = Utils.indexToObject(getParent(id), getUserAttribute(id, "siloTriggerNode"));
	instance.silo.maxZScale = getUserAttribute(id, "siloMaxZScale");
	instance.silo.minZScale = getUserAttribute(id, "siloMinZScale");
	
	instance.silo.triggerTransBackup = {getTranslation(instance.silo.trigger)};
	setTranslation(instance.silo.trigger, 0,-10,0);
	setVisibility(instance.silo.plane, false);
	
	addTrigger(instance.silo.trigger, "SiloGetCallback", instance);
	
	
	instance.folie = {};
	instance.folie.plane = Utils.indexToObject(getParent(id), getUserAttribute(id, "folieIndex"));
	
	setVisibility(instance.folie.index, false);
	
	instance.movingIndex = Utils.indexToObject(getParent(id), getUserAttribute(id, "movingIndex"));
	
	
	instance.trailerInTrigger = {};	
		
		
	instance.trailerID = nil;
	instance.fillLevel = 0;
	instance.silo.maxFillLevel = instance.silage.maxFillLevel;
	
	instance.tipTrailer = false;
	instance.attachableID = nil;
	
	instance.playerInRange = false;
	instance.siloState = 0;
	
	
	instance:moveSilagePlane(instance.fillLevel);
	instance:moveSiloPlane(instance.fillLevel);
	instance:updateState();
	
	--instance.firstTimeRun = true;
	
	return instance;		
	

	
end;

function MapBGASilo:loadMap(name)
end;

function MapBGASilo:deleteMap()
end;

function MapBGASilo:delete()
end;

function MapBGASilo:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MapBGASilo:keyEvent(unicode, sym, modifier, isDown)
end;

function MapBGASilo:update(dt)



	if table.getn(self.trailerInTrigger) > 0 then
		if self.siloState == 0 then		
			if g_currentMission.controlPlayer then
				g_currentMission:onEnterVehicle(g_currentMission.vehicleInMountRange);
			end;
			
			local tip = false;
			if InputBinding.hasEvent(InputBinding.ATTACH) then
				self.tipTrailer = not self.tipTrailer;
				tip = true;
			end;
			for i=1, table.getn(self.trailerInTrigger) do
				if self.tipTrailer then			
					self.fillLevel = self.fillLevel - self.trailerInTrigger[i].lastFillDelta;
					self:moveSilagePlane(self.fillLevel);
					if self.fillLevel >= self.silage.maxFillLevel and (self.trailerInTrigger[i].tipState == Trailer.TIPSTATE_OPENING or self.trailerInTrigger[i].tipState == Trailer.TIPSTATE_OPEN) then
						self.trailerInTrigger[i]:toggleTipState();
					end;					
				end;	
				if tip and self.trailerInTrigger[i].tipState ~= Trailer.TIPSTATE_CLOSING then
					--self.tipTrailer = not self.tipTrailer;
					self.trailerInTrigger[i]:toggleTipState();
				end;	
				g_currentMission.trailerInTipRange = self.trailerInTrigger[i];		
				g_currentMission.currentVehicle.doRefuel = true; -- not allowed to Togglevehicle
				g_currentMission.allowSteerableMoving = true;
				g_currentMission.fixedCamera = false;
			end;
		end;
	else
		if self.siloState == 0 and self.fillLevel >= self.silage.maxFillLevel then
			self.siloState = 1;
			self:updateState(self);
		end;
		self.tipTrailer = false;
	end;
	
	
	if self.playerInRange and self.fillLevel > 0 then
		if InputBinding.hasEvent(InputBinding.BGASILOSTATE) then
			self.siloState = self.siloState +1;
			
			self:updateState(self);
			
			--[[if self.siloState == 1 then -- folieplane
				setVisibility(self.silage.plane, true);
				setVisibility(self.folie.plane, true);
				setVisibility(self.silage.plane, true);
			elseif self.siloState == 2 then -- siloplane		
				setVisibility(self.silage.plane, false);
				setVisibility(self.folie.plane, false);
				setVisibility(self.silo.plane, true);
				setTranslation(self.silo.trigger, unpack(self.silo.triggerTransBackup));		
				setRigidBodyType(self.silage.plane, "NoRigidBody");
				if self.fillLevel <= 0 then
					self.siloState = 0;
					self.fillLevel = 0;
					self:moveSilagePlane(self.fillLevel);
				end;
				self.silo.maxFillLevel = self.fillLevel;
				self:moveSiloPlane(self.fillLevel)
			else	-- silageplane
				setVisibility(self.silo.plane, false);
				setTranslation(self.silo.trigger, 0,-10,0);
				setVisibility(self.folie.plane, false);
				setRigidBodyType(self.silage.plane, "Kinematic");		
				setVisibility(self.silage.plane, true);
			end;]]
			
			--[[if self.siloState == 2 then
				self.silo.maxFillLevel = self.fillLevel;
				self:moveSiloPlane(self.fillLevel)
			end;]]
			
		end;
		if self.siloState == 3 then
			self.siloState = 0;
			self:updateState();
		end;
	end;	
	
	if self.fillLevel > self.silage.maxFillLevel then
		self.fillLevel = self.silage.maxFillLevel;
	end;
	
	if self.fillLevel <= 0 and self.siloState == 2 then
		self.siloState = 0;
		self.fillLevel = 0;
		self:updateState();
		self:moveSilagePlane(self.fillLevel);
	end;
	
	if self.attachableID ~= nil then			
		if self.attachableID.fillTypes ~= nil and self.attachableID.setFillLevel ~= nil and self.attachableID.fillLevel ~= nil then
			if self.attachableID:allowFillType(FruitUtil.FRUITTYPE_SILO, true) then
				if self.attachableID.fillLevel < self.attachableID.capacity then
					local deltaFillLevel = dt*2;
					self.attachableID:setFillLevel(self.attachableID.fillLevel+deltaFillLevel, FruitUtil.FRUITTYPE_SILO);
					self.fillLevel = self.fillLevel - deltaFillLevel;
					self:moveSiloPlane(self.fillLevel);
				end;
			end;
		end;
	end;
	

	
end;

function MapBGASilo:moveSilagePlane(fillLevel)

	local m = (self.silage.moveMaxY - self.silage.moveMinY) / self.silage.maxFillLevel;
	local xPos, yPos, zPos = getTranslation(self.movingIndex);
	setTranslation(self.movingIndex, xPos, m*fillLevel + self.silage.moveMinY, zPos);	

end;

function MapBGASilo:moveSiloPlane(fillLevel)

	local m = (self.silo.maxZScale - self.silo.minZScale) / self.silo.maxFillLevel;
	local xScale, yScale, zScale = getScale(self.silo.plane);
	setScale(self.silo.plane, xScale, yScale, (m*fillLevel + self.silo.minZScale)^2);	

	local xw,yw,zw = getWorldTranslation(self.silo.triggerNode);	
	local xl,yl,zl = worldToLocal(getParent(getParent(self.silo.trigger)), xw, yw, zw);
	setTranslation(getParent(self.silo.trigger), xl, yl, zl);
	
end;

function MapBGASilo:updateState()

	if self.siloState == 1 then -- folieplane
		setVisibility(self.silage.plane, true);
		setVisibility(self.folie.plane, true);
		setVisibility(self.silage.plane, true);
	elseif self.siloState == 2 then -- siloplane		
		setVisibility(self.silage.plane, false);
		setVisibility(self.folie.plane, false);
		setVisibility(self.silo.plane, true);
		setTranslation(self.silo.trigger, unpack(self.silo.triggerTransBackup));		
		setRigidBodyType(self.silage.plane, "NoRigidBody");
		if self.fillLevel <= 0 then
			self.siloState = 0;
			self.fillLevel = 0;
			self:moveSilagePlane(self.fillLevel);
		end;
		self.silo.maxFillLevel = self.fillLevel;
		self:moveSiloPlane(self.fillLevel)
	else	-- silageplane
		setVisibility(self.silo.plane, false);
		setTranslation(self.silo.trigger, 0,-10,0);
		setVisibility(self.folie.plane, false);
		setRigidBodyType(self.silage.plane, "Kinematic");		
		setVisibility(self.silage.plane, true);
	end;	

end;

function MapBGASilo:draw()

	if table.getn(self.trailerInTrigger) > 0 then
		for i=1, table.getn(self.trailerInTrigger) do	
			if self.siloState == 0 and self.trailerInTrigger[i].attacherVehicle == g_currentMission.currentVehicle then
				--g_currentMission:addExtraPrintText("Abkippen: Taste "..InputBinding.getButtonKeyName(InputBinding.FRONTLOADER_TILT_NEG));
				g_currentMission.hudTipperOverlay:render();
				g_currentMission:addExtraPrintText("FillLevel: "..math.floor(self.fillLevel));
			end;			
		end;
	end;
	
	if self.playerInRange then
		if self.siloState == 0 and self.fillLevel > 0 then
			g_currentMission:addExtraPrintText("Silo zudecken: Taste "..InputBinding.getButtonKeyName(InputBinding.BGASILOSTATE));
		elseif self.siloState == 1 then
			g_currentMission:addExtraPrintText("Silo aufdecken: Taste "..InputBinding.getButtonKeyName(InputBinding.BGASILOSTATE));
		elseif self.siloState == 2 then
			g_currentMission:addExtraPrintText("Silage einfahren: Taste "..InputBinding.getButtonKeyName(InputBinding.BGASILOSTATE));
		end;
	end;

	
end;

function MapBGASilo:SiloTipCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
    
	if onEnter then
		local trailer = g_currentMission.objectToTrailer[otherId];
		local enterTable = false;
		if trailer ~= nil then
			if trailer.setFillLevel ~= nil and self.silage.allowFillTypes[trailer.currentFillType] and trailer.tipAnimCharSet ~= 0 then			
				for i=0, table.getn(self.trailerInTrigger) do
					if self.trailerInTrigger[i] ~= trailer then					
						enterTable = true;
					else
						enterTable = false;
					end;
				end;
			end;
		end;		
		if enterTable then
			table.insert(self.trailerInTrigger, trailer)
			--print("enter name - ", getName(otherId));
			--print("tablesize - ", table.getn(self.trailerInTrigger));
		end;		
		if otherId == Player.rootNode then
			self.playerInRange = true;
		end;
		--[[if self.firstTimeRun then
			self:moveSilagePlane(self.fillLevel+100);
			self.firstTimeRun = false;
		end;]]
	elseif onLeave then
		local trailer = g_currentMission.objectToTrailer[otherId];
		if trailer ~= nil then
			for i=0, table.getn(self.trailerInTrigger) do
				if self.trailerInTrigger[i] == trailer then
					if self.trailerInTrigger[i].tipState == Trailer.TIPSTATE_OPENING or self.trailerInTrigger[i].tipState == Trailer.TIPSTATE_OPEN then
						self.trailerInTrigger[i]:toggleTipState();
					end;
					table.remove(self.trailerInTrigger, i)
					g_currentMission.trailerInTipRange = nil;
					--print("leave name - ", getName(otherId));
					--print("tablesize - ", table.getn(self.trailerInTrigger));
				end;
			end;
		end;	
		if otherId == Player.rootNode then
			self.playerInRange = false;
		end;
	end;
	
end;

function MapBGASilo:SiloGetCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
    if onEnter then
		--print("otherId", otherId)
		--print("name", getName(otherId))
		if otherId ~= nil then	
			local attachable = g_currentMission.objectToTrailer[otherId];
			if attachable ~= nil then
				if attachable.isShovel ~= nil or attachable.getFromSilo ~= nil then
					self.attachableID = attachable;	
				end;
			end;
		end;		
	elseif onLeave then
		if otherId ~= nil then	
			local attachable = g_currentMission.objectToTrailer[otherId];
			if attachable ~= nil then
				if attachable.isShovel ~= nil or attachable.getFromSilo ~= nil then
					self.attachableID = nil;
				end;
			end;
		end;					
	end;	
	
end;

